/*
 * Copyright 2009 Cedric Priscal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <jni.h>

#include "mango_standard.h"
#include "gpio_helper.h"
#include "android/log.h"
#include "buffDump.h"
#include "i2c-dev.h"
#define BMP085_CHIP_ID_REG      0xD0
#define BMP085_VERSION_REG      0xD1
#define SLAVE_ADDRESS_RD 0x77  //BMP085

#define BMP085_CTRL_REG         0xF4
#define BMP085_TEMP_MEASUREMENT     0x2E


static const char *TAG="serial_port";

#define LOGI(fmt, args...) __android_log_print(ANDROID_LOG_INFO,  TAG, fmt, ##args)
#define LOGD(fmt, args...) __android_log_print(ANDROID_LOG_DEBUG, TAG, fmt, ##args)
#define LOGE(fmt, args...) __android_log_print(ANDROID_LOG_ERROR, TAG, fmt, ##args)

static speed_t getBaudrate(jint baudrate)
{
	switch(baudrate) {
	case 0: return B0;
	case 50: return B50;
	case 75: return B75;
	case 110: return B110;
	case 134: return B134;
	case 150: return B150;
	case 200: return B200;
	case 300: return B300;
	case 600: return B600;
	case 1200: return B1200;
	case 1800: return B1800;
	case 2400: return B2400;
	case 4800: return B4800;
	case 9600: return B9600;
	case 19200: return B19200;
	case 38400: return B38400;
	case 57600: return B57600;
	case 115200: return B115200;
	case 230400: return B230400;
	case 460800: return B460800;
	case 500000: return B500000;
	case 576000: return B576000;
	case 921600: return B921600;
	case 1000000: return B1000000;
	case 1152000: return B1152000;
	case 1500000: return B1500000;
	case 2000000: return B2000000;
	case 2500000: return B2500000;
	case 3500000: return B3500000;
	case 4000000: return B4000000;
	default: return -1;
	}
}

/*
 * Class:     cedric_serial_SerialPort
 * Method:    open
 * Signature: (Ljava/lang/String;)V
 */

JNIEXPORT jobject JNICALL Java_android_gpio_SerialPort_open
  (JNIEnv *env, jobject thiz, jstring path, jint baudrate)
{
	int fd;
	char *buf;
	speed_t speed;
	jobject mFileDescriptor;

	/* Check arguments */
	{
		system("su");
		system("ls -al"); //testo
		//sprintf(buf,"chmod 777 %s",path);
		system(buf);
		speed = getBaudrate(baudrate);
		if (speed == -1) {
			/* TODO: throw an exception */
			LOGE("Invalid baudrate");
			return NULL;
		}
	}

	/* Opening device */
	{
		jboolean iscopy;
		const char *path_utf = (*env)->GetStringUTFChars(env, path, &iscopy);
		LOGD("Opening serial port %s", path_utf);
		//fd = open(path_utf, O_RDWR | O_DIRECT | O_SYNC);		
		fd = open(path_utf, O_RDWR | O_SYNC);
		LOGD("open() fd = %d", fd);
		(*env)->ReleaseStringUTFChars(env, path, path_utf);
		if (fd == -1)
		{
			/* Throw an exception */
			LOGE("Cannot open port");
			/* TODO: throw an exception */
			return NULL;
		}
	}

	/* Configure device */
	{
		struct termios cfg;
		LOGD("Configuring serial port");
		if (tcgetattr(fd, &cfg))
		{
			LOGE("tcgetattr() failed");
			close(fd);
			/* TODO: throw an exception */
			return NULL;
		}

		cfmakeraw(&cfg);
		cfsetispeed(&cfg, speed);
		cfsetospeed(&cfg, speed);

		if (tcsetattr(fd, TCSANOW, &cfg))
		{
			LOGE("tcsetattr() failed");
			close(fd);
			/* TODO: throw an exception */
			return NULL;
		}
	}

	/* Create a corresponding file descriptor */
	{
		jclass cFileDescriptor = (*env)->FindClass(env, "java/io/FileDescriptor");
		jmethodID iFileDescriptor = (*env)->GetMethodID(env, cFileDescriptor, "<init>", "()V");
		jfieldID descriptorID = (*env)->GetFieldID(env, cFileDescriptor, "descriptor", "I");
		mFileDescriptor = (*env)->NewObject(env, cFileDescriptor, iFileDescriptor);
		(*env)->SetIntField(env, mFileDescriptor, descriptorID, (jint)fd);
	}

	return mFileDescriptor;
}

/*
 * Class:     cedric_serial_SerialPort
 * Method:    close
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_android_gpio_SerialPort_close
  (JNIEnv *env, jobject thiz)
{
	jclass SerialPortClass = (*env)->GetObjectClass(env, thiz);
	jclass FileDescriptorClass = (*env)->FindClass(env, "java/io/FileDescriptor");

	jfieldID mFdID = (*env)->GetFieldID(env, SerialPortClass, "mFd", "Ljava/io/FileDescriptor;");
	jfieldID descriptorID = (*env)->GetFieldID(env, FileDescriptorClass, "descriptor", "I");

	jobject mFd = (*env)->GetObjectField(env, thiz, mFdID);
	jint descriptor = (*env)->GetIntField(env, mFd, descriptorID);

	LOGD("close(fd = %d)", descriptor);
	close(descriptor);
}
/*
 * Class:     gpio
 * Method:    export
 * Signature: ()V
 */
/*
JNIEXPORT void JNICALL Java_android_serialport_SerialPort_gpioexport
  (JNIEnv *env, jobject thiz,jint gpio, jstring direction)
{
	jboolean iscopy;
	const char *direction_utf = (*env)->GetStringUTFChars(env, direction, &iscopy);
	LOGD("Opening serial port %s", direction_utf);


	gpio_export(gpio);
	if(strcmp(direction_utf,"in")==0)
	{
		gpio_dir_out(gpio);
	}

	else
	{
		gpio_dir_in(gpio);
	}

}
*/



int i2c_write_func(unsigned int addr,unsigned int command , unsigned value )
{

	int count, RW;
	unsigned char *i2c_buffer;
	unsigned char i2c_ptr[2];

	int dev_i2c;
	int err;



	LOGD("Write from I2C  at 0x%X, cmd=0x%X, value=0x%X\n", addr, command, value);
	i2c_buffer = malloc(sizeof(char) * 10);

	if(i2c_buffer == NULL) {
		LOGD("STATUS: Couldn't malloc buffer.\n");
		return(-1);
	}

	// Open and read i2c
	if ((dev_i2c = open("/dev/i2c-0", O_RDWR)) < 0) {
		LOGD("Cannot Open I2C Master Device\n");
	    exit(1);
	}

	if (ioctl(dev_i2c, I2C_SLAVE_FORCE, addr) < 0) {
		LOGD("Cannot set I2C address\n");
	    exit(1);
	}

	err=i2c_smbus_write_byte_data(dev_i2c,command,value);
	if(err<0)
		{
		LOGD(" %s error\n",__func__);
		return -1;
		}
	return(0);
}
int i2c_crl_read_func(unsigned int addr, unsigned int command)
{

	int count, value,RW;
	unsigned char *i2c_buffer;
	unsigned char i2c_ptr[2];
	unsigned argRead, bytesRead;
	int dev_i2c;
	unsigned int data,buff;



	i2c_buffer = malloc(sizeof(char) * 32);

	if(i2c_buffer == NULL) {
		LOGD("STATUS: Couldn't malloc buffer.\n");
		return(-1);
	}

	// Open and read i2c
	if ((dev_i2c = open("/dev/i2c-0", O_RDWR)) < 0) {
		LOGD("Cannot Open I2C Master Device\n");
	    exit(1);
	}

	if (ioctl(dev_i2c, I2C_SLAVE_FORCE, addr) < 0) {
      		LOGD("Cannot set I2C address\n");
	    exit(1);
	}
	//i2c_smbus_write_byte(dev_i2c,0x90); //Write Config
	//i2c_smbus_write_byte(dev_i2c,command); //Write Config
	//buff = i2c_smbus_read_word(dev_i2c);
	buff=i2c_smbus_read_byte_data(dev_i2c,command);
	LOGD("buff=0x%x \n",buff);
	data = buff & 0xFF;
	//LOGD("data=0x%x \n",data);
	//data = data << 8;
	//data = data | ((buff >> 8) & 0xFF);
	//LOGD("read data2=%d \n",data);





	return(data);
}


JNIEXPORT void JNICALL Java_android_gpio_sample_GpioActivity_gpioexport
  (JNIEnv *env, jobject thiz)
{

	gpio_export(GPIO_OUT0);
	gpio_dir_out(GPIO_OUT0);
	LOGD("gpio_out 0");
	gpio_export(GPIO_OUT1);
	gpio_dir_out(GPIO_OUT1);
	LOGD("gpio_out 1");


	//LOGD("gpioexport 0");
	//gpio_export(GPIO_IN0);
	//gpio_dir_in(GPIO_IN0);

}

JNIEXPORT void JNICALL Java_android_gpio_sample_GpioActivity_gpioout0
  (JNIEnv *env, jobject thiz,jint out)
{
	gpio_set_value(GPIO_OUT0,out);
	LOGD("GPIO_OUT0 %d ",out);
}
JNIEXPORT void JNICALL Java_android_gpio_sample_GpioActivity_gpioout1
  (JNIEnv *env, jobject thiz,jint out)
{
	gpio_set_value(GPIO_OUT1,out);
	LOGD("GPIO_OUT1 %d ",out);
}

/*
JNIEXPORT jint JNICALL Java_android_gpio_sample_GpioActivity_gpioin1
  (JNIEnv *env, jobject thiz)
{
	jint value;
	value=gpio_get_value(GPIO_IN1);
	LOGD("GPIO_IN1 %d ",value);
	return value;
}
*/
JNIEXPORT jint JNICALL Java_android_gpio_sample_GpioActivity_i2cread
  (JNIEnv *env, jobject thiz)
{
	jint value;
	value=i2c_crl_read_func(SLAVE_ADDRESS_RD,BMP085_CHIP_ID_REG);
	//value=i2c_crl_read_func(0x77,0xD0);
	LOGD("read data2=%d \n",value);
	return value;
}


JNIEXPORT void JNICALL Java_android_gpio_sample_GpioActivity_i2cwrite
  (JNIEnv *env, jobject thiz)
{
	i2c_write_func(SLAVE_ADDRESS_RD,BMP085_CTRL_REG,BMP085_TEMP_MEASUREMENT);
	
}

