/*
 *  tslib/tests/ts_calibrate.c
 *
 *  Copyright (C) 2001 Russell King.
 *
 * This file is placed under the GPL.  Please see the file
 * COPYING for more details.
 *
 * $Id: ts_calibrate.c,v 1.8 2004/10/19 22:01:27 dlowder Exp $
 *
 * Basic test program for touchscreen library.
 */
#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <linux/kd.h>
#include <linux/vt.h>
#include <linux/fb.h>
#include <jni.h>

#include "tslib.h"

#include "fbutils.h"
#include "testutils.h"
#include "mango_standard.h"
#include "android/log.h"

static const char *TAG="TSCAL";

#define LOGI(fmt, args...) __android_log_print(ANDROID_LOG_INFO,  TAG, fmt, ##args)
#define LOGD(fmt, args...) __android_log_print(ANDROID_LOG_DEBUG, TAG, fmt, ##args)
#define LOGE(fmt, args...) __android_log_print(ANDROID_LOG_ERROR, TAG, fmt, ##args)

static int palette [] =
{
	0x000000, 0xffe080, 0xffffff, 0xe0c0a0
};
#define NR_COLORS (sizeof (palette) / sizeof (palette [0]))

typedef struct {
	int x[5], xfb[5];
	int y[5], yfb[5];
	int a[7];
} calibration;

static void sig(int sig)
{
	close_framebuffer ();
	fflush (stderr);
	printf ("signal %d caught\n", sig);
	LOGD ("signal %d caught\n", sig);
	fflush (stdout);
	exit (1);
}

int perform_calibration(calibration *cal) {
	int j;
	float n, x, y, x2, y2, xy, z, zx, zy;
	float det, a, b, c, e, f, i;
	float scaling = 65536.0;

// Get sums for matrix
	n = x = y = x2 = y2 = xy = 0;
	for(j=0;j<5;j++) {
		n += 1.0;
		x += (float)cal->x[j];
		y += (float)cal->y[j];
		x2 += (float)(cal->x[j]*cal->x[j]);
		y2 += (float)(cal->y[j]*cal->y[j]);
		xy += (float)(cal->x[j]*cal->y[j]);
	}

// Get determinant of matrix -- check if determinant is too small
	det = n*(x2*y2 - xy*xy) + x*(xy*y - x*y2) + y*(x*xy - y*x2);
	if(det < 0.1 && det > -0.1) {
		printf("ts_calibrate: determinant is too small -- %f\n",det);
		LOGD("ts_calibrate: determinant is too small -- %f\n",det);
		return 0;
	}

// Get elements of inverse matrix
	a = (x2*y2 - xy*xy)/det;
	b = (xy*y - x*y2)/det;
	c = (x*xy - y*x2)/det;
	e = (n*y2 - y*y)/det;
	f = (x*y - n*xy)/det;
	i = (n*x2 - x*x)/det;

// Get sums for x calibration
	z = zx = zy = 0;
	for(j=0;j<5;j++) {
		z += (float)cal->xfb[j];
		zx += (float)(cal->xfb[j]*cal->x[j]);
		zy += (float)(cal->xfb[j]*cal->y[j]);
	}

// Now multiply out to get the calibration for framebuffer x coord
	cal->a[0] = (int)((a*z + b*zx + c*zy)*(scaling));
	cal->a[1] = (int)((b*z + e*zx + f*zy)*(scaling));
	cal->a[2] = (int)((c*z + f*zx + i*zy)*(scaling));

	printf("%f %f %f\n",(a*z + b*zx + c*zy),
				(b*z + e*zx + f*zy),
				(c*z + f*zx + i*zy));
	LOGD("%f %f %f\n",(a*z + b*zx + c*zy),
				(b*z + e*zx + f*zy),
				(c*z + f*zx + i*zy));


// Get sums for y calibration
	z = zx = zy = 0;
	for(j=0;j<5;j++) {
		z += (float)cal->yfb[j];
		zx += (float)(cal->yfb[j]*cal->x[j]);
		zy += (float)(cal->yfb[j]*cal->y[j]);
	}

// Now multiply out to get the calibration for framebuffer y coord
	cal->a[3] = (int)((a*z + b*zx + c*zy)*(scaling));
	cal->a[4] = (int)((b*z + e*zx + f*zy)*(scaling));
	cal->a[5] = (int)((c*z + f*zx + i*zy)*(scaling));

	printf("%f %f %f\n",(a*z + b*zx + c*zy),
				(b*z + e*zx + f*zy),
				(c*z + f*zx + i*zy));
	LOGD("%f %f %f\n",(a*z + b*zx + c*zy),
				(b*z + e*zx + f*zy),
				(c*z + f*zx + i*zy));

// If we got here, we're OK, so assign scaling to a[6] and return
	cal->a[6] = (int)scaling;
	LOGD("Scaling : %d %d",(int)scaling,cal->a[6]);
	return 1;
/*	
// This code was here originally to just insert default values
	for(j=0;j<7;j++) {
		c->a[j]=0;
	}
	c->a[1] = c->a[5] = c->a[6] = 1;
	return 1;
*/

}

static int get_sample (struct tsdev *ts, calibration *cal,
			int index, int x, int y, char *name)
{
	static int last_x = -1, last_y;
	int ret;

#if 1   //by crazyboy 20140214 by treego FixIT  
	if (last_x != -1) {
#define NR_STEPS 10
		int dx = ((x - last_x) << 16) / NR_STEPS;
		int dy = ((y - last_y) << 16) / NR_STEPS;
		int i;
		last_x <<= 16;
		last_y <<= 16;
		for (i = 0; i < NR_STEPS; i++) {
	//		put_cross (last_x >> 16, last_y >> 16, 2 | XORMODE);
	//		LOGD("Debug : %s %d\n",__func__,__LINE__);
			usleep (1000);
	//		put_cross (last_x >> 16, last_y >> 16, 2 | XORMODE);
	//		LOGD("Debug : %s %d\n",__func__,__LINE__);
			last_x += dx;
			last_y += dy;
		}
	}
#endif

//	put_cross(x, y, 2 | XORMODE);
	ret = getxy (ts, &cal->x [index], &cal->y [index]);
//	put_cross(x, y, 2 | XORMODE);

	last_x = cal->xfb [index] = x;
	last_y = cal->yfb [index] = y;

	LOGD("%s : X = %4d Y = %4d ret = %d\n", name, cal->x [index], cal->y [index], ret);

	return ret;
	
}

static void clearbuf(struct tsdev *ts)
{
	int fd = ts_fd(ts);
	fd_set fdset;
	struct timeval tv;
	int nfds;
	struct ts_sample sample;

	while (1) {
		FD_ZERO(&fdset);
		FD_SET(fd, &fdset);

		tv.tv_sec = 0;
		tv.tv_usec = 0;

		nfds = select(fd + 1, &fdset, NULL, NULL, &tv);
		if (nfds == 0) break;

		if (ts_read_raw(ts, &sample, 1) < 0) {
			perror("ts_read");
			exit(1);
		}
	}
}

//int main()
#if 1
JNIEXPORT jint JNICALL Java_android_gpio_sample_TScalActivity_tscal
		  (JNIEnv *env, jobject thiz)
#else
#if 0
JNIEXPORT jint JNICALL Java_android_gpio_sample_MainActivity_tscal
 		  (JNIEnv *env, jobject thiz)
#else 
JNIEXPORT jint JNICALL Java_android_gpio_sample_Tscalthread_tscal
 		  (JNIEnv *env, jobject thiz)
#endif
#endif
{
	struct tsdev *ts;
	calibration cal;
	int cal_fd;
	char cal_buffer[256];
	char *tsdevice = NULL;
	char *calfile = NULL;
	unsigned int i;
	int ret;

	memset(cal_buffer, 0 ,sizeof(cal_buffer));

	signal(SIGSEGV, sig);
	signal(SIGINT, sig);
	signal(SIGTERM, sig);

	if( (tsdevice = getenv("TSLIB_TSDEVICE")) != NULL ) {
		ts = ts_open(tsdevice,0);
	} else {
		if (!(ts = ts_open("/dev/input/event1", 0)))
			//ts = ts_open("/dev/touchscreen/ucb1x00", 0);
			ts = ts_open("/dev/input/event1", 0);
	}

	if (!ts) {
		perror("ts_open");
		exit(1);
	}
	if (ts_config(ts)) {
		perror("ts_config");
		exit(1);
	}

#if 0
	if (open_framebuffer()) {
		close_framebuffer();
		LOGD("ERROR DA Close framebuffer\n ");
		exit(1);
	}
	LOGD("TS CALIBRATION START - 5\n ");

	for (i = 0; i < NR_COLORS; i++)
		setcolor (i, palette [i]);

	put_string_center (xres / 2, yres / 4,
			   "TSLIB calibration utility", 1);
	put_string_center (xres / 2, yres / 4 + 20,
			   "Touch crosshair to calibrate", 2);
#endif

	xres = LCD_XRES;
	yres = LCD_YRES; 

	LOGD("xres = %d, yres = %d\n", xres, yres);

	// Clear the buffer
	clearbuf(ts);
	ret = get_sample (ts, &cal, 0, 50,        50,        "Top left");
	if(ret == 2)
	{
		LOGD("Get X,Y Value Error!! \n");
		return ret;

	}
	clearbuf(ts);
	ret = get_sample (ts, &cal, 1, xres - 50, 50,        "Top right");
	if(ret == 2)
	{
		LOGD("Get X,Y Value Error!! \n");
		return ret;

	}
	clearbuf(ts);
	ret = get_sample (ts, &cal, 2, xres - 50, yres - 50, "Bot right");
	if(ret == 2)
	{
		LOGD("Get X,Y Value Error!! \n");
		return ret;

	}
	clearbuf(ts);
	ret = get_sample (ts, &cal, 3, 50,        yres - 50, "Bot left");
	if(ret == 2)
	{
		LOGD("Get X,Y Value Error!! \n");
		return ret;

	}
	clearbuf(ts);
	ret = get_sample (ts, &cal, 4, xres / 2,  yres / 2,  "Center");
	if(ret == 2)
	{
		LOGD("Get X,Y Value Error!! \n");
		return ret;

	}

	if (perform_calibration (&cal)) {
		LOGD ("Calibration constants: %d %d %d %d %d %d %d",cal.a[0], cal.a[1], cal.a[2], cal.a[3],
														  cal.a[4], cal.a[5],cal.a[6]);
		for (i = 0; i < 7; i++) LOGD("%d ", cal.a [i]);
		LOGD("\n");
		LOGD("CRZ  TS_POINTERCAL=%s\n",TS_POINTERCAL); 
		if ((calfile = getenv("TSLIB_CALIBFILE")) != NULL) {
			cal_fd = open (calfile, O_CREAT | O_RDWR,
			               S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
		} else {
			cal_fd = open (TS_POINTERCAL,O_CREAT| O_RDWR, 
			              S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
	//		cal_fd = open (TS_POINTERCAL,O_RDWR | O_SYNC);
			               
		}
#if 1
		system("/dev/null > /data/pointercal");
		sprintf (cal_buffer,"%d %d %d %d %d %d %d",
			 cal.a[1], cal.a[2], cal.a[0],
			 cal.a[4], cal.a[5], cal.a[3], cal.a[6]);
		write (cal_fd, cal_buffer, strlen (cal_buffer) + 1);
#else 

		sprintf (cal_buffer,"%d %d %d %d %d %d",
			 cal.a[1], cal.a[2], cal.a[0],
			 cal.a[4], cal.a[5], cal.a[3]);

		write (cal_fd, cal_buffer, strlen(cal_buffer) + 1);

		LOGD("strlen = %d",strlen(cal_buffer) + 1);
#endif

		close (cal_fd);
        ret = 0;
		LOGD("CRZ  TS_POINTERCAL END \n"); 
	} else {
		LOGD("Calibration failed.\n");
		ret = -1;
	}

//	close_framebuffer();
	return ret;
}
